package sun.security.jgss;

import java.util.HashSet;
import java.util.Set;
import java.util.Map;
import sun.security.jgss.spi.GSSNameSpi;
import java.util.Vector;
import org.ietf.jgss.GSSCredential;
import java.util.Enumeration;
import sun.security.jgss.spnego.SpNegoCredElement;
import org.ietf.jgss.GSSName;
import org.ietf.jgss.GSSException;
import org.ietf.jgss.Oid;
import sun.security.jgss.spi.GSSCredentialSpi;
import java.util.Hashtable;
import com.sun.security.jgss.ExtendedGSSCredential;

public class GSSCredentialImpl implements ExtendedGSSCredential
{
    private GSSManagerImpl gssManager;
    private boolean destroyed;
    private Hashtable<GSSCredentialImpl.SearchKey, GSSCredentialSpi> hashtable;
    private GSSCredentialSpi tempCred;
    
    GSSCredentialImpl(GSSManagerImpl gssManagerImpl, int n) throws GSSException {
        this(gssManagerImpl, null, 0, (Oid[])null, n);
    }
    
    GSSCredentialImpl(GSSManagerImpl gssManagerImpl, GSSName gssName, int n, Oid default_MECH_OID, int n2) throws GSSException {
        this.gssManager = null;
        this.destroyed = false;
        this.hashtable = null;
        this.tempCred = null;
        if (default_MECH_OID == null) {
            default_MECH_OID = ProviderList.DEFAULT_MECH_OID;
        }
        this.init(gssManagerImpl);
        this.add(gssName, n, n, default_MECH_OID, n2);
    }
    
    GSSCredentialImpl(GSSManagerImpl gssManagerImpl, GSSName gssName, int n, Oid[] mechs, int n2) throws GSSException {
        this.gssManager = null;
        this.destroyed = false;
        this.hashtable = null;
        this.tempCred = null;
        this.init(gssManagerImpl);
        boolean n3 = false;
        if (mechs == null) {
            mechs = gssManagerImpl.getMechs();
            n3 = true;
        }
        for (int i = 0; i < mechs.length; ++i) {
            try {
                this.add(gssName, n, n, mechs[i], n2);
            }
            catch (GSSException obj) {
                if (!(n3 == 0)) {
                    GSSUtil.debug("Ignore " + obj + " while acquring cred for " + mechs[i]);
                }
                else {
                    throw obj;
                }
            }
        }
        if (this.hashtable.size() == 0 || n2 != this.getUsage()) {
            throw new GSSException(13);
        }
        return;
    }
    
    public GSSCredentialImpl(GSSManagerImpl gssManagerImpl, GSSCredentialSpi gssCredentialSpi) throws GSSException {
        this.gssManager = null;
        this.destroyed = false;
        this.hashtable = null;
        this.tempCred = null;
        this.init(gssManagerImpl);
        int n = 2;
        if (gssCredentialSpi.isInitiatorCredential()) {
            if (gssCredentialSpi.isAcceptorCredential()) {
                n = 0;
            }
            else {
                n = 1;
            }
        }
        GSSCredentialImpl.SearchKey key = new GSSCredentialImpl.SearchKey(gssCredentialSpi.getMechanism(), n);
        this.tempCred = gssCredentialSpi;
        this.hashtable.put(key, this.tempCred);
        if (!GSSUtil.isSpNegoMech(gssCredentialSpi.getMechanism())) {
            this.hashtable.put(new GSSCredentialImpl.SearchKey(GSSUtil.GSS_SPNEGO_MECH_OID, n), new SpNegoCredElement(gssCredentialSpi));
        }
    }
    
    void init(GSSManagerImpl gssManager) {
        this.gssManager = gssManager;
        this.hashtable = new Hashtable(gssManager.getMechs().length);
    }
    
    @Override
    public void dispose() throws GSSException {
        if (!this.destroyed) {
            Enumeration<GSSCredentialSpi> elements = this.hashtable.elements();
            while (elements.hasMoreElements()) {
                ((GSSCredentialSpi)elements.nextElement()).dispose();
            }
            this.destroyed = true;
        }
    }
    
    @Override
    public GSSCredential impersonate(GSSName gssName) throws GSSException {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        Oid mechanism = this.tempCred.getMechanism();
        GSSCredentialSpi impersonate = this.tempCred.impersonate((gssName == null) ? null : ((GSSNameImpl)gssName).getElement(mechanism));
        return (impersonate == null) ? null : new GSSCredentialImpl(this.gssManager, impersonate);
    }
    
    @Override
    public GSSName getName() throws GSSException {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        return GSSNameImpl.wrapElement(this.gssManager, this.tempCred.getName());
    }
    
    @Override
    public GSSName getName(Oid default_MECH_OID) throws GSSException {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        if (default_MECH_OID == null) {
            default_MECH_OID = ProviderList.DEFAULT_MECH_OID;
        }
        GSSCredentialSpi gssCredentialSpi = (GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, 1));
        if (gssCredentialSpi == null) {
            gssCredentialSpi = (GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, 2));
        }
        if (gssCredentialSpi == null) {
            gssCredentialSpi = (GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, 0));
        }
        if (gssCredentialSpi == null) {
            throw new GSSExceptionImpl(2, default_MECH_OID);
        }
        return GSSNameImpl.wrapElement(this.gssManager, gssCredentialSpi.getName());
    }
    
    @Override
    public int getRemainingLifetime() throws GSSException {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        int n = Integer.MAX_VALUE;
        Enumeration<GSSCredentialImpl.SearchKey> keys = this.hashtable.keys();
        while (keys.hasMoreElements()) {
            GSSCredentialImpl.SearchKey key = (GSSCredentialImpl.SearchKey)keys.nextElement();
            GSSCredentialSpi gssCredentialSpi = (GSSCredentialSpi)this.hashtable.get(key);
            int n2;
            if (key.getUsage() == 1) {
                n2 = gssCredentialSpi.getInitLifetime();
            }
            else if (key.getUsage() == 2) {
                n2 = gssCredentialSpi.getAcceptLifetime();
            }
            else {
                int initLifetime = gssCredentialSpi.getInitLifetime();
                int acceptLifetime = gssCredentialSpi.getAcceptLifetime();
                n2 = ((initLifetime < acceptLifetime) ? initLifetime : acceptLifetime);
            }
            if (n <= n2) {
                continue;
            }
            n = n2;
        }
        return n;
    }
    
    @Override
    public int getRemainingInitLifetime(Oid default_MECH_OID) throws GSSException {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        boolean n = false;
        int n2 = 0;
        if (default_MECH_OID == null) {
            default_MECH_OID = ProviderList.DEFAULT_MECH_OID;
        }
        GSSCredentialSpi gssCredentialSpi = (GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, 1));
        if (gssCredentialSpi != null) {
            n = true;
            if (n2 < gssCredentialSpi.getInitLifetime()) {
                n2 = gssCredentialSpi.getInitLifetime();
            }
        }
        GSSCredentialSpi gssCredentialSpi2 = (GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, 0));
        if (gssCredentialSpi2 != null) {
            n = true;
            if (n2 < gssCredentialSpi2.getInitLifetime()) {
                n2 = gssCredentialSpi2.getInitLifetime();
            }
        }
        if (n == 0) {
            throw new GSSExceptionImpl(2, default_MECH_OID);
        }
        return n2;
    }
    
    @Override
    public int getRemainingAcceptLifetime(Oid default_MECH_OID) throws GSSException {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        boolean n = false;
        int n2 = 0;
        if (default_MECH_OID == null) {
            default_MECH_OID = ProviderList.DEFAULT_MECH_OID;
        }
        GSSCredentialSpi gssCredentialSpi = (GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, 2));
        if (gssCredentialSpi != null) {
            n = true;
            if (n2 < gssCredentialSpi.getAcceptLifetime()) {
                n2 = gssCredentialSpi.getAcceptLifetime();
            }
        }
        GSSCredentialSpi gssCredentialSpi2 = (GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, 0));
        if (gssCredentialSpi2 != null) {
            n = true;
            if (n2 < gssCredentialSpi2.getAcceptLifetime()) {
                n2 = gssCredentialSpi2.getAcceptLifetime();
            }
        }
        if (n == 0) {
            throw new GSSExceptionImpl(2, default_MECH_OID);
        }
        return n2;
    }
    
    @Override
    public int getUsage() throws GSSException {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        boolean n = false;
        boolean n2 = false;
        Enumeration<GSSCredentialImpl.SearchKey> keys = this.hashtable.keys();
        while (keys.hasMoreElements()) {
            GSSCredentialImpl.SearchKey searchKey = (GSSCredentialImpl.SearchKey)keys.nextElement();
            if (searchKey.getUsage() == 1) {
                n = true;
            }
            else if (searchKey.getUsage() == 2) {
                n2 = true;
            }
            else {
                return 0;
            }
        }
        if (!(n == 0)) {
            if (!(n2 == 0)) {
                return 0;
            }
            return 1;
        }
        else {
            return 2;
        }
    }
    
    @Override
    public int getUsage(Oid default_MECH_OID) throws GSSException {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        boolean n = false;
        boolean n2 = false;
        if (default_MECH_OID == null) {
            default_MECH_OID = ProviderList.DEFAULT_MECH_OID;
        }
        if ((GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, 1)) != null) {
            n = true;
        }
        if ((GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, 2)) != null) {
            n2 = true;
        }
        if ((GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, 0)) != null) {
            n = true;
            n2 = true;
        }
        if (n && n2) {
            return 0;
        }
        if (!(n == 0)) {
            return 1;
        }
        if (!(n2 == 0)) {
            return 2;
        }
        throw new GSSExceptionImpl(2, default_MECH_OID);
    }
    
    @Override
    public Oid[] getMechs() throws GSSException {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        Vector vector = new Vector(this.hashtable.size());
        Enumeration<GSSCredentialImpl.SearchKey> keys = this.hashtable.keys();
        while (keys.hasMoreElements()) {
            vector.addElement(((GSSCredentialImpl.SearchKey)keys.nextElement()).getMech());
        }
        return (Oid[])vector.toArray(new Oid[0]);
    }
    
    @Override
    public void add(GSSName gssName, int n, int n2, Oid default_MECH_OID, int n3) throws GSSException {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        if (default_MECH_OID == null) {
            default_MECH_OID = ProviderList.DEFAULT_MECH_OID;
        }
        GSSCredentialImpl.SearchKey o = new GSSCredentialImpl.SearchKey(default_MECH_OID, n3);
        if (this.hashtable.containsKey(o)) {
            throw new GSSExceptionImpl(17, "Duplicate element found: " + getElementStr(default_MECH_OID, n3));
        }
        GSSNameSpi gssNameSpi = (gssName == null) ? null : ((GSSNameImpl)gssName).getElement(default_MECH_OID);
        this.tempCred = this.gssManager.getCredentialElement(gssNameSpi, n, n2, default_MECH_OID, n3);
        if (this.tempCred != null) {
            if (n3 == 0 && (!this.tempCred.isAcceptorCredential() || !this.tempCred.isInitiatorCredential())) {
                int n4;
                int n5;
                if (!this.tempCred.isInitiatorCredential()) {
                    n4 = 2;
                    n5 = 1;
                }
                else {
                    n4 = 1;
                    n5 = 2;
                }
                this.hashtable.put(new GSSCredentialImpl.SearchKey(default_MECH_OID, n4), this.tempCred);
                this.tempCred = this.gssManager.getCredentialElement(gssNameSpi, n, n2, default_MECH_OID, n5);
                this.hashtable.put(new GSSCredentialImpl.SearchKey(default_MECH_OID, n5), this.tempCred);
            }
            else {
                this.hashtable.put(o, this.tempCred);
            }
        }
        return;
    }
    
    @Override
    public boolean equals(Object o) {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        return this == o || (!(o instanceof GSSCredentialImpl) && false);
    }
    
    @Override
    public int hashCode() {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        return 1;
    }
    
    public GSSCredentialSpi getElement(Oid default_MECH_OID, boolean b) throws GSSException {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        GSSCredentialSpi gssCredentialSpi;
        if (default_MECH_OID == null) {
            default_MECH_OID = ProviderList.DEFAULT_MECH_OID;
            gssCredentialSpi = (GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, b ? 1 : 2));
            if (gssCredentialSpi == null) {
                gssCredentialSpi = (GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, 0));
                if (gssCredentialSpi == null) {
                    Object[] array = this.hashtable.entrySet().toArray();
                    int i = 0;
                    while (i < array.length) {
                        gssCredentialSpi = (GSSCredentialSpi)((Map.Entry)array[i]).getValue();
                        if (gssCredentialSpi.isInitiatorCredential() == b) {
                            break;
                        }
                        ++i;
                    }
                }
            }
        }
        else {
            GSSCredentialImpl.SearchKey key;
            if (b) {
                key = new GSSCredentialImpl.SearchKey(default_MECH_OID, 1);
            }
            else {
                key = new GSSCredentialImpl.SearchKey(default_MECH_OID, 2);
            }
            gssCredentialSpi = (GSSCredentialSpi)this.hashtable.get(key);
            if (gssCredentialSpi == null) {
                gssCredentialSpi = (GSSCredentialSpi)this.hashtable.get(new GSSCredentialImpl.SearchKey(default_MECH_OID, 0));
            }
        }
        if (gssCredentialSpi == null) {
            throw new GSSExceptionImpl(13, "No credential found for: " + getElementStr(default_MECH_OID, b ? 1 : 2));
        }
        return gssCredentialSpi;
    }
    
    Set<GSSCredentialSpi> getElements() {
        HashSet set = new HashSet(this.hashtable.size());
        Enumeration<GSSCredentialSpi> elements = this.hashtable.elements();
        while (elements.hasMoreElements()) {
            set.add((E)elements.nextElement());
        }
        return set;
    }
    
    private static String getElementStr(Oid oid, int n) {
        String string = oid.toString();
        String s;
        if (n == 1) {
            s = string.concat(" usage: Initiate");
        }
        else if (n == 2) {
            s = string.concat(" usage: Accept");
        }
        else {
            s = string.concat(" usage: Initiate and Accept");
        }
        return s;
    }
    
    @Override
    public String toString() {
        if (this.destroyed) {
            throw new IllegalStateException("This credential is no longer valid");
        }
        StringBuffer sb = new StringBuffer("[GSSCredential: ");
        Object[] array = this.hashtable.entrySet().toArray();
        for (int i = 0; i < array.length; ++i) {
            try {
                sb.append('\n');
                GSSCredentialSpi gssCredentialSpi = (GSSCredentialSpi)((Map.Entry)array[i]).getValue();
                sb.append(gssCredentialSpi.getName());
                sb.append(' ');
                sb.append(gssCredentialSpi.getMechanism());
                sb.append(gssCredentialSpi.isInitiatorCredential() ? " Initiate" : "");
                sb.append(gssCredentialSpi.isAcceptorCredential() ? " Accept" : "");
                sb.append(" [");
                sb.append(gssCredentialSpi.getClass());
                sb.append(']');
            }
            catch (GSSException ex) {}
        }
        sb.append(']');
        return sb.toString();
    }
    
    static class SearchKey
    {
        private Oid mechOid;
        private int usage;
        
        public SearchKey(Oid mechOid, int usage) {
            this.mechOid = null;
            this.usage = 0;
            this.mechOid = mechOid;
            this.usage = usage;
        }
        
        public Oid getMech() {
            return this.mechOid;
        }
        
        public int getUsage() {
            return this.usage;
        }
        
        @Override
        public boolean equals(Object o) {
            if (!(o instanceof GSSCredentialImpl.SearchKey)) {
                return false;
            }
            GSSCredentialImpl.SearchKey searchKey = (GSSCredentialImpl.SearchKey)o;
            return this.mechOid.equals(searchKey.mechOid) && this.usage == searchKey.usage;
        }
        
        @Override
        public int hashCode() {
            return this.mechOid.hashCode();
        }
    }
}
